"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.toProjectReferences = void 0;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var comment_json_1 = __importDefault(require("comment-json"));
var npm_1 = require("./manager/npm");
var yarn_1 = require("./manager/yarn");
var assert_1 = __importDefault(require("assert"));
exports.toProjectReferences = function (options) {
    var _a;
    var plugins = (_a = options.plugins) !== null && _a !== void 0 ? _a : [npm_1.plugin, yarn_1.plugin];
    var pluginImplementations = plugins.map(function (plugin) { return plugin(options); });
    // use first plugin
    var supportPlugin = pluginImplementations.find(function (plugin) {
        return plugin.supports();
    });
    if (!supportPlugin) {
        throw new Error("Not found supported plugin");
    }
    var relativeName = function (pkgPath) {
        return path_1.default.relative(options.rootDir, pkgPath);
    };
    var allPackages = supportPlugin.getAllPackages();
    var errors = [];
    allPackages.forEach(function (packageInfo) {
        var _a, _b, _c;
        var tsconfigFilePath = (_b = (_a = options.tsConfigPathFinder) === null || _a === void 0 ? void 0 : _a.call(options, packageInfo.location)) !== null && _b !== void 0 ? _b : path_1.default.join(packageInfo.location, "tsconfig.json");
        if (!fs_1.default.existsSync(tsconfigFilePath)) {
            // Skip has not tsconfig.json
            return;
        }
        var tsconfigJSON = comment_json_1.default.parse(fs_1.default.readFileSync(tsconfigFilePath, "utf-8"));
        var references = supportPlugin.getDependencies(packageInfo.packageJSON);
        var newProjectReferences = references
            .map(function (reference) {
            var absolutePathOrNull = supportPlugin.resolve(reference);
            if (!absolutePathOrNull) {
                return;
            }
            if (!path_1.default.isAbsolute(absolutePathOrNull)) {
                throw new Error("Plugin#resolve should return absolute path: " + absolutePathOrNull + ", plugin: " + supportPlugin);
            }
            if (packageInfo.location === absolutePathOrNull) {
                var selfName = relativeName(packageInfo.location);
                errors.push(new Error("[" + selfName + "] Self dependencies is something wrong: " + selfName + " refer to " + relativeName(absolutePathOrNull)));
            }
            return {
                path: path_1.default.relative(packageInfo.location, absolutePathOrNull)
            };
        })
            .filter(function (r) { return Boolean(r); });
        var currentProjectReferences = (_c = tsconfigJSON["references"]) !== null && _c !== void 0 ? _c : [];
        if (options.checkOnly) {
            // check
            try {
                // create pure object for compare
                var cleanCurrentProjectReferences = JSON.parse(JSON.stringify(comment_json_1.default.parse(comment_json_1.default.stringify(currentProjectReferences), undefined, true)));
                assert_1.default.deepStrictEqual(cleanCurrentProjectReferences, newProjectReferences);
            }
            catch (error) {
                var selfName = relativeName(packageInfo.location);
                errors.push(new Error("[" + selfName + "] " + error.message));
            }
            return;
        }
        else {
            // update
            if (errors.length === 0) {
                tsconfigJSON["references"] = newProjectReferences;
                fs_1.default.writeFileSync(tsconfigFilePath, comment_json_1.default.stringify(tsconfigJSON, null, 2), "utf-8");
            }
        }
    });
    if (errors.length > 0) {
        return {
            ok: false,
            aggregateError: {
                message: "Can not update Project References, because found " + errors.length + " errors",
                errors: errors
            }
        };
    }
    return {
        ok: true
    };
};
//# sourceMappingURL=index.js.map