import fs from "fs";
import path from "path";
import commentJSON from "comment-json";
import { plugin as npmPlugin } from "./manager/npm";
import { plugin as yarnPlugin } from "./manager/yarn";
import assert from "assert";
export var toProjectReferences = function (options) {
    var _a;
    var plugins = (_a = options.plugins) !== null && _a !== void 0 ? _a : [npmPlugin, yarnPlugin];
    var pluginImplementations = plugins.map(function (plugin) { return plugin(options); });
    // use first plugin
    var supportPlugin = pluginImplementations.find(function (plugin) {
        return plugin.supports();
    });
    if (!supportPlugin) {
        throw new Error("Not found supported plugin");
    }
    var relativeName = function (pkgPath) {
        return path.relative(options.rootDir, pkgPath);
    };
    var allPackages = supportPlugin.getAllPackages();
    var errors = [];
    allPackages.forEach(function (packageInfo) {
        var _a, _b, _c;
        var tsconfigFilePath = (_b = (_a = options.tsConfigPathFinder) === null || _a === void 0 ? void 0 : _a.call(options, packageInfo.location)) !== null && _b !== void 0 ? _b : path.join(packageInfo.location, "tsconfig.json");
        if (!fs.existsSync(tsconfigFilePath)) {
            // Skip has not tsconfig.json
            return;
        }
        var tsconfigJSON = commentJSON.parse(fs.readFileSync(tsconfigFilePath, "utf-8"));
        var references = supportPlugin.getDependencies(packageInfo.packageJSON);
        var newProjectReferences = references
            .map(function (reference) {
            var absolutePathOrNull = supportPlugin.resolve(reference);
            if (!absolutePathOrNull) {
                return;
            }
            if (!path.isAbsolute(absolutePathOrNull)) {
                throw new Error("Plugin#resolve should return absolute path: " + absolutePathOrNull + ", plugin: " + supportPlugin);
            }
            if (packageInfo.location === absolutePathOrNull) {
                var selfName = relativeName(packageInfo.location);
                errors.push(new Error("[" + selfName + "] Self dependencies is something wrong: " + selfName + " refer to " + relativeName(absolutePathOrNull)));
            }
            return {
                path: path.relative(packageInfo.location, absolutePathOrNull)
            };
        })
            .filter(function (r) { return Boolean(r); });
        var currentProjectReferences = (_c = tsconfigJSON["references"]) !== null && _c !== void 0 ? _c : [];
        if (options.checkOnly) {
            // check
            try {
                // create pure object for compare
                var cleanCurrentProjectReferences = JSON.parse(JSON.stringify(commentJSON.parse(commentJSON.stringify(currentProjectReferences), undefined, true)));
                assert.deepStrictEqual(cleanCurrentProjectReferences, newProjectReferences);
            }
            catch (error) {
                var selfName = relativeName(packageInfo.location);
                errors.push(new Error("[" + selfName + "] " + error.message));
            }
            return;
        }
        else {
            // update
            if (errors.length === 0) {
                tsconfigJSON["references"] = newProjectReferences;
                fs.writeFileSync(tsconfigFilePath, commentJSON.stringify(tsconfigJSON, null, 2), "utf-8");
            }
        }
    });
    if (errors.length > 0) {
        return {
            ok: false,
            aggregateError: {
                message: "Can not update Project References, because found " + errors.length + " errors",
                errors: errors
            }
        };
    }
    return {
        ok: true
    };
};
//# sourceMappingURL=index.js.map